"""
    HW Mux Reservation System

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 2.1.6
    Generated by: https://openapi-generator.tech
"""


import unittest

import hwmux_client
from hwmux_client.api.groups_api import GroupsApi  # noqa: E501


class TestGroupsApi(unittest.TestCase):
    """GroupsApi unit test stubs"""

    def setUp(self):
        self.api = GroupsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_groups_available_list(self):
        """Test case for groups_available_list

        """
        pass

    def test_groups_create(self):
        """Test case for groups_create

        """
        pass

    def test_groups_create_with_devices_create(self):
        """Test case for groups_create_with_devices_create

        """
        pass

    def test_groups_destroy(self):
        """Test case for groups_destroy

        """
        pass

    def test_groups_list(self):
        """Test case for groups_list

        """
        pass

    def test_groups_my_list(self):
        """Test case for groups_my_list

        """
        pass

    def test_groups_partial_update(self):
        """Test case for groups_partial_update

        """
        pass

    def test_groups_release_by_name_update(self):
        """Test case for groups_release_by_name_update

        """
        pass

    def test_groups_release_update(self):
        """Test case for groups_release_update

        """
        pass

    def test_groups_reserve_by_name_update(self):
        """Test case for groups_reserve_by_name_update

        """
        pass

    def test_groups_reserve_update(self):
        """Test case for groups_reserve_update

        """
        pass

    def test_groups_retrieve(self):
        """Test case for groups_retrieve

        """
        pass

    def test_groups_update(self):
        """Test case for groups_update

        """
        pass


if __name__ == '__main__':
    unittest.main()



from . import Configuration, ApiClient
from .apis import (
    GroupsApi,
    DevicesApi,
    SitesApi,
    RoomsApi,
    PartsApi,
    PartFamiliesApi,
    LabelsApi,
    LogsApi,
    SchemaApi,
    ReservationsApi,
)
from .models import ReservationRequest, ReservationSessionSerializerReadOnly
from urllib3.util.retry import Retry
from typing import Optional
from threading import Thread
from time import sleep
from datetime import datetime, timezone
import logging

__all__ = [
    "HwMuxApi",
    "ReservationNotSatisfiedException",
    "SatisfiedReservationWatchdog",
]


class ReservationNotSatisfiedException(Exception):
    """
    Raised from HwMuxApicreate_reservation() when a reservation is not
    satisfied and raise_error_if_not_satisfied=True
    """


def ApiDecorator(apiClass):
    class NewApiClass(apiClass):
        @classmethod
        def list_all(cls, method, *args, **kwargs):
            return HwMuxApi.list_all(method, *args, **kwargs)

    return NewApiClass


class HwMuxApi:
    """
    HwMuxApi is a wrapper around the hwmux_client library that provides access to all public APIs.
    """

    def __init__(
        self,
        user_token: Optional[str] = None,
        server_url: Optional[str] = None,
        retries: Optional[Retry] = None,
    ):
        config = Configuration(host=server_url)
        if user_token is not None:
            config = Configuration(
                host=server_url,
                api_key={"tokenAuth": user_token},
                api_key_prefix={"tokenAuth": "Token"},
                discard_unknown_keys=True,
            )
        config.retries = retries
        self.client = ApiClient(config)
        self.apis = {}
        for api in [
            GroupsApi,
            DevicesApi,
            SitesApi,
            RoomsApi,
            PartsApi,
            PartFamiliesApi,
            LabelsApi,
            LogsApi,
            SchemaApi,
            ReservationsApi,
        ]:
            self.apis[api.__name__] = ApiDecorator(api)(self.client)

    @classmethod
    def list_all(cls, method, *args, **kwargs):
        all_items = []
        page = 0
        while True:
            page += 1
            kwargs["page"] = page
            items = method(*args, **kwargs)
            all_items.extend(items.results)
            if items.next is None:
                break
        return all_items

    def get_location_url(self, site, room, location_id):
        url = f"{self.client.configuration.host}/api/sites/{site}/rooms/{room}/locations/{location_id}/".replace(
            "https", "http"
        )
        return url

    @property
    def groups_api(self) -> GroupsApi:
        return self.apis["GroupsApi"]

    @property
    def devices_api(self) -> DevicesApi:
        return self.apis["DevicesApi"]

    @property
    def sites_api(self) -> SitesApi:
        return self.apis["SitesApi"]

    @property
    def rooms_api(self) -> RoomsApi:
        return self.apis["RoomsApi"]

    @property
    def parts_api(self) -> PartsApi:
        return self.apis["PartsApi"]

    @property
    def part_families_api(self) -> PartFamiliesApi:
        return self.apis["PartFamiliesApi"]

    @property
    def labels_api(self) -> LabelsApi:
        return self.apis["LabelsApi"]

    @property
    def logs_api(self) -> LogsApi:
        return self.apis["LogsApi"]

    @property
    def schema_api(self) -> SchemaApi:
        return self.apis["SchemaApi"]

    @property
    def reservations_api(self) -> ReservationsApi:
        return self.apis["ReservationsApi"]

    def create_reservation(
        self, request: ReservationRequest, raise_error_if_not_satisfied=False
    ):
        """
        Create a reservation and wait until it is satisfied.
        Args:
            request: reservation request
            raise_error_if_not_satisfied: if True, raise ReservationNotSatisfiedException if reservation is not satisfied

        Returns:
            SatisfiedReservationWatchdog object that periodically renews the
            reservation lease in a background thread and can be used to release the reservation.

        """
        logger = logging.getLogger(__name__)
        reservation: ReservationSessionSerializerReadOnly = (
            self.reservations_api.reservations_create(reservation_request=request)
        )
        while not reservation.is_satisfied:
            if raise_error_if_not_satisfied:
                self.reservations_api.reservations_cancel_update(reservation.id)
                raise ReservationNotSatisfiedException("Reservation not satisfied")
            interval = min(
                30,
                int(seconds_until_lease_expires(reservation) / 2),
            )
            logger.info(f"Reservation not satisfied, waiting {interval} seconds...")
            sleep(interval)
            reservation = self.reservations_api.reservations_update(reservation.id)
        return SatisfiedReservationWatchdog(self, reservation)


class SatisfiedReservationWatchdog:
    """
    A SatisfiedReservationWatchdog object is returned from HwMuxApi.create_reservation()
    It contains a background thread that periodically renews the reservation lease and a
    release() method that cancels the background thread and releases the reservation.
    """

    default_watchdog_max_interval = 30

    def __init__(
        self, hwmux_api: HwMuxApi, reservation: ReservationSessionSerializerReadOnly
    ):
        """
        Creates a new SatisfiedReservationWatchdog object and starts the background thread.
        Args:
            hwmux_api: instance of HwMuxApi
            reservation_id: reservation id
        """
        self.stop_watchdog = False
        self.reservation = reservation
        self.hwmux_api = hwmux_api
        self.watchdog_thread = Thread(target=self.watchdog_thread)
        self.watchdog_thread.daemon = True
        self.watchdog_thread.start()

    @property
    def reservation_id(self):
        return self.reservation.id

    def release(self):
        """
        Releases the reservation and stops the background thread.
        """
        self.stop_watchdog = True
        self.watchdog_thread.join()
        self.hwmux_api.reservations_api.reservations_release_update(self.reservation_id)

    def wait_for_watchdog_interval(self, interval: int):
        """
        Waits for the watchdog interval to expire or until stop_watchdog is set to True.
        """
        start = datetime.now()
        while (
            datetime.now() - start
        ).total_seconds() < interval and not self.stop_watchdog:
            sleep(0.1)

    def watchdog_thread(self):
        """
        Background thread that periodically renews the reservation lease.
        """
        logger = logging.getLogger(__name__)
        while not self.stop_watchdog:
            interval = min(
                self.default_watchdog_max_interval,
                int(seconds_until_lease_expires(self.reservation) / 2),
            )
            self.wait_for_watchdog_interval(interval)
            if not self.stop_watchdog:
                logger.debug("Renewing reservation lease")
                self.reservation = self.hwmux_api.reservations_api.reservations_update(
                    self.reservation_id
                )


def seconds_until_lease_expires(reservation: ReservationSessionSerializerReadOnly):
    return (reservation.t_lease_expires - datetime.now(timezone.utc)).total_seconds()



"""
    HW Mux Reservation System

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 2.13.0
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from hwmux_client.api_client import ApiClient, Endpoint as _Endpoint
from hwmux_client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from hwmux_client.model.paginated_device_group_list import PaginatedDeviceGroupList
from hwmux_client.model.paginated_device_serializer_public_list import PaginatedDeviceSerializerPublicList
from hwmux_client.model.paginated_label_list import PaginatedLabelList
from hwmux_client.model.paginated_permission_group_list import PaginatedPermissionGroupList
from hwmux_client.model.patched_permission_group import PatchedPermissionGroup
from hwmux_client.model.patched_resource_permissions import PatchedResourcePermissions
from hwmux_client.model.permission_group import PermissionGroup
from hwmux_client.model.resource_permissions import ResourcePermissions
from hwmux_client.model.user import User


class PermissionsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client
        self.permissions_groups_create_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/',
                'operation_id': 'permissions_groups_create',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'permission_group',
                ],
                'required': [
                    'permission_group',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'permission_group':
                        (PermissionGroup,),
                },
                'attribute_map': {
                },
                'location_map': {
                    'permission_group': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_destroy_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/',
                'operation_id': 'permissions_groups_destroy',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                ],
                'required': [
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'name_or_id',
                ]
            },
            root_map={
                'validations': {
                    ('name_or_id',): {

                        'regex': {
                            'pattern': r'[^\/]+',  # noqa: E501
                        },
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_device_groups_create_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/device_groups/',
                'operation_id': 'permissions_groups_device_groups_create',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'resource_permissions',
                ],
                'required': [
                    'name_or_id',
                    'resource_permissions',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'resource_permissions':
                        ([ResourcePermissions],),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_device_groups_destroy_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/device_groups/{id}/',
                'operation_id': 'permissions_groups_device_groups_destroy',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                ],
                'required': [
                    'id',
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_device_groups_list_endpoint = _Endpoint(
            settings={
                'response_type': (PaginatedDeviceGroupList,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/device_groups/',
                'operation_id': 'permissions_groups_device_groups_list',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'page',
                    'perms',
                ],
                'required': [
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                    'perms',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('perms',): {

                        "ADD": "add",
                        "ADD,DELETE": "add,delete",
                        "CHANGE": "change",
                        "CHANGE,ADD": "change,add",
                        "CHANGE,ADD,DELETE": "change,add,delete",
                        "CHANGE,DELETE": "change,delete",
                        "DELETE": "delete",
                        "VIEW": "view",
                        "VIEW,ADD": "view,add",
                        "VIEW,ADD,DELETE": "view,add,delete",
                        "VIEW,CHANGE": "view,change",
                        "VIEW,CHANGE,ADD": "view,change,add",
                        "VIEW,CHANGE,ADD,DELETE": "view,change,add,delete",
                        "VIEW,CHANGE,DELETE": "view,change,delete",
                        "VIEW,DELETE": "view,delete"
                    },
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'page':
                        (int,),
                    'perms':
                        (str,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                    'page': 'page',
                    'perms': 'perms',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'page': 'query',
                    'perms': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_device_groups_partial_update_endpoint = _Endpoint(
            settings={
                'response_type': (ResourcePermissions,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/device_groups/{id}/',
                'operation_id': 'permissions_groups_device_groups_partial_update',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                    'patched_resource_permissions',
                ],
                'required': [
                    'id',
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                    'patched_resource_permissions':
                        (PatchedResourcePermissions,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                    'patched_resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_device_groups_update_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/device_groups/{id}/',
                'operation_id': 'permissions_groups_device_groups_update',
                'http_method': 'PUT',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                    'resource_permissions',
                ],
                'required': [
                    'id',
                    'name_or_id',
                    'resource_permissions',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                    'resource_permissions':
                        (ResourcePermissions,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                    'resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_devices_create_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/devices/',
                'operation_id': 'permissions_groups_devices_create',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'resource_permissions',
                ],
                'required': [
                    'name_or_id',
                    'resource_permissions',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'resource_permissions':
                        ([ResourcePermissions],),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_devices_destroy_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/devices/{id}/',
                'operation_id': 'permissions_groups_devices_destroy',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                ],
                'required': [
                    'id',
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_devices_list_endpoint = _Endpoint(
            settings={
                'response_type': (PaginatedDeviceSerializerPublicList,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/devices/',
                'operation_id': 'permissions_groups_devices_list',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'page',
                    'perms',
                ],
                'required': [
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                    'perms',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('perms',): {

                        "ADD": "add",
                        "ADD,DELETE": "add,delete",
                        "CHANGE": "change",
                        "CHANGE,ADD": "change,add",
                        "CHANGE,ADD,DELETE": "change,add,delete",
                        "CHANGE,DELETE": "change,delete",
                        "DELETE": "delete",
                        "VIEW": "view",
                        "VIEW,ADD": "view,add",
                        "VIEW,ADD,DELETE": "view,add,delete",
                        "VIEW,CHANGE": "view,change",
                        "VIEW,CHANGE,ADD": "view,change,add",
                        "VIEW,CHANGE,ADD,DELETE": "view,change,add,delete",
                        "VIEW,CHANGE,DELETE": "view,change,delete",
                        "VIEW,DELETE": "view,delete"
                    },
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'page':
                        (int,),
                    'perms':
                        (str,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                    'page': 'page',
                    'perms': 'perms',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'page': 'query',
                    'perms': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_devices_partial_update_endpoint = _Endpoint(
            settings={
                'response_type': (ResourcePermissions,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/devices/{id}/',
                'operation_id': 'permissions_groups_devices_partial_update',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                    'patched_resource_permissions',
                ],
                'required': [
                    'id',
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                    'patched_resource_permissions':
                        (PatchedResourcePermissions,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                    'patched_resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_devices_update_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/devices/{id}/',
                'operation_id': 'permissions_groups_devices_update',
                'http_method': 'PUT',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                    'resource_permissions',
                ],
                'required': [
                    'id',
                    'name_or_id',
                    'resource_permissions',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                    'resource_permissions':
                        (ResourcePermissions,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                    'resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_labels_create_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/labels/',
                'operation_id': 'permissions_groups_labels_create',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'resource_permissions',
                ],
                'required': [
                    'name_or_id',
                    'resource_permissions',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'resource_permissions':
                        ([ResourcePermissions],),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_labels_destroy_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/labels/{id}/',
                'operation_id': 'permissions_groups_labels_destroy',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                ],
                'required': [
                    'id',
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_labels_list_endpoint = _Endpoint(
            settings={
                'response_type': (PaginatedLabelList,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/labels/',
                'operation_id': 'permissions_groups_labels_list',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'page',
                    'perms',
                ],
                'required': [
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                    'perms',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('perms',): {

                        "ADD": "add",
                        "ADD,DELETE": "add,delete",
                        "CHANGE": "change",
                        "CHANGE,ADD": "change,add",
                        "CHANGE,ADD,DELETE": "change,add,delete",
                        "CHANGE,DELETE": "change,delete",
                        "DELETE": "delete",
                        "VIEW": "view",
                        "VIEW,ADD": "view,add",
                        "VIEW,ADD,DELETE": "view,add,delete",
                        "VIEW,CHANGE": "view,change",
                        "VIEW,CHANGE,ADD": "view,change,add",
                        "VIEW,CHANGE,ADD,DELETE": "view,change,add,delete",
                        "VIEW,CHANGE,DELETE": "view,change,delete",
                        "VIEW,DELETE": "view,delete"
                    },
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'page':
                        (int,),
                    'perms':
                        (str,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                    'page': 'page',
                    'perms': 'perms',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'page': 'query',
                    'perms': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_labels_partial_update_endpoint = _Endpoint(
            settings={
                'response_type': (ResourcePermissions,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/labels/{id}/',
                'operation_id': 'permissions_groups_labels_partial_update',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                    'patched_resource_permissions',
                ],
                'required': [
                    'id',
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                    'patched_resource_permissions':
                        (PatchedResourcePermissions,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                    'patched_resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_labels_update_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/labels/{id}/',
                'operation_id': 'permissions_groups_labels_update',
                'http_method': 'PUT',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'name_or_id',
                    'resource_permissions',
                ],
                'required': [
                    'id',
                    'name_or_id',
                    'resource_permissions',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (int,),
                    'name_or_id':
                        (str,),
                    'resource_permissions':
                        (ResourcePermissions,),
                },
                'attribute_map': {
                    'id': 'id',
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'id': 'path',
                    'name_or_id': 'path',
                    'resource_permissions': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_list_endpoint = _Endpoint(
            settings={
                'response_type': (PaginatedPermissionGroupList,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/',
                'operation_id': 'permissions_groups_list',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'ordering',
                    'page',
                    'search',
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'ordering':
                        (str,),
                    'page':
                        (int,),
                    'search':
                        (str,),
                },
                'attribute_map': {
                    'ordering': 'ordering',
                    'page': 'page',
                    'search': 'search',
                },
                'location_map': {
                    'ordering': 'query',
                    'page': 'query',
                    'search': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_partial_update_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/',
                'operation_id': 'permissions_groups_partial_update',
                'http_method': 'PATCH',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'patched_permission_group',
                ],
                'required': [
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'name_or_id',
                ]
            },
            root_map={
                'validations': {
                    ('name_or_id',): {

                        'regex': {
                            'pattern': r'[^\/]+',  # noqa: E501
                        },
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'patched_permission_group':
                        (PatchedPermissionGroup,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'patched_permission_group': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_retrieve_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/',
                'operation_id': 'permissions_groups_retrieve',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                ],
                'required': [
                    'name_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'name_or_id',
                ]
            },
            root_map={
                'validations': {
                    ('name_or_id',): {

                        'regex': {
                            'pattern': r'[^\/]+',  # noqa: E501
                        },
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client
        )
        self.permissions_groups_update_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/',
                'operation_id': 'permissions_groups_update',
                'http_method': 'PUT',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'permission_group',
                ],
                'required': [
                    'name_or_id',
                    'permission_group',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                    'name_or_id',
                ]
            },
            root_map={
                'validations': {
                    ('name_or_id',): {

                        'regex': {
                            'pattern': r'[^\/]+',  # noqa: E501
                        },
                    },
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'permission_group':
                        (PermissionGroup,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'permission_group': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_users_create_endpoint = _Endpoint(
            settings={
                'response_type': (PermissionGroup,),
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/users/',
                'operation_id': 'permissions_groups_users_create',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'user',
                ],
                'required': [
                    'name_or_id',
                    'user',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'user':
                        ([User],),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'user': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json',
                    'application/x-www-form-urlencoded',
                    'multipart/form-data'
                ]
            },
            api_client=api_client
        )
        self.permissions_groups_users_destroy_endpoint = _Endpoint(
            settings={
                'response_type': None,
                'auth': [
                    'cookieAuth',
                    'tokenAuth'
                ],
                'endpoint_path': '/api/permissions/groups/{name_or_id}/users/{username_or_id}/',
                'operation_id': 'permissions_groups_users_destroy',
                'http_method': 'DELETE',
                'servers': None,
            },
            params_map={
                'all': [
                    'name_or_id',
                    'username_or_id',
                ],
                'required': [
                    'name_or_id',
                    'username_or_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'name_or_id':
                        (str,),
                    'username_or_id':
                        (str,),
                },
                'attribute_map': {
                    'name_or_id': 'name_or_id',
                    'username_or_id': 'username_or_id',
                },
                'location_map': {
                    'name_or_id': 'path',
                    'username_or_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [],
                'content_type': [],
            },
            api_client=api_client
        )

    def permissions_groups_create(
        self,
        permission_group,
        **kwargs
    ):
        """permissions_groups_create  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_create(permission_group, async_req=True)
        >>> result = thread.get()

        Args:
            permission_group (PermissionGroup):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['permission_group'] = \
            permission_group
        return self.permissions_groups_create_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_destroy(
        self,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_destroy  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_destroy(name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_destroy_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_device_groups_create(
        self,
        name_or_id,
        resource_permissions,
        **kwargs
    ):
        """permissions_groups_device_groups_create  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_device_groups_create(name_or_id, resource_permissions, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):
            resource_permissions ([ResourcePermissions]):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['resource_permissions'] = \
            resource_permissions
        return self.permissions_groups_device_groups_create_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_device_groups_destroy(
        self,
        id,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_device_groups_destroy  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_device_groups_destroy(id, name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device group.
            name_or_id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_device_groups_destroy_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_device_groups_list(
        self,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_device_groups_list  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_device_groups_list(name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):

        Keyword Args:
            page (int): A page number within the paginated result set.. [optional]
            perms (str): Permission types. Only entities with all permissions defined here will be fetched. Defaults to all permissions.. [optional] if omitted the server will use the default value of "view,change,add,delete"
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PaginatedDeviceGroupList
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_device_groups_list_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_device_groups_partial_update(
        self,
        id,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_device_groups_partial_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_device_groups_partial_update(id, name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device group.
            name_or_id (str):

        Keyword Args:
            patched_resource_permissions (PatchedResourcePermissions): [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            ResourcePermissions
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_device_groups_partial_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_device_groups_update(
        self,
        id,
        name_or_id,
        resource_permissions,
        **kwargs
    ):
        """permissions_groups_device_groups_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_device_groups_update(id, name_or_id, resource_permissions, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device group.
            name_or_id (str):
            resource_permissions (ResourcePermissions):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['resource_permissions'] = \
            resource_permissions
        return self.permissions_groups_device_groups_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_devices_create(
        self,
        name_or_id,
        resource_permissions,
        **kwargs
    ):
        """permissions_groups_devices_create  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_devices_create(name_or_id, resource_permissions, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):
            resource_permissions ([ResourcePermissions]):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['resource_permissions'] = \
            resource_permissions
        return self.permissions_groups_devices_create_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_devices_destroy(
        self,
        id,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_devices_destroy  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_devices_destroy(id, name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device.
            name_or_id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_devices_destroy_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_devices_list(
        self,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_devices_list  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_devices_list(name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):

        Keyword Args:
            page (int): A page number within the paginated result set.. [optional]
            perms (str): Permission types. Only entities with all permissions defined here will be fetched. Defaults to all permissions.. [optional] if omitted the server will use the default value of "view,change,add,delete"
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PaginatedDeviceSerializerPublicList
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_devices_list_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_devices_partial_update(
        self,
        id,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_devices_partial_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_devices_partial_update(id, name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device.
            name_or_id (str):

        Keyword Args:
            patched_resource_permissions (PatchedResourcePermissions): [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            ResourcePermissions
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_devices_partial_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_devices_update(
        self,
        id,
        name_or_id,
        resource_permissions,
        **kwargs
    ):
        """permissions_groups_devices_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_devices_update(id, name_or_id, resource_permissions, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device.
            name_or_id (str):
            resource_permissions (ResourcePermissions):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['resource_permissions'] = \
            resource_permissions
        return self.permissions_groups_devices_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_labels_create(
        self,
        name_or_id,
        resource_permissions,
        **kwargs
    ):
        """permissions_groups_labels_create  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_labels_create(name_or_id, resource_permissions, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):
            resource_permissions ([ResourcePermissions]):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['resource_permissions'] = \
            resource_permissions
        return self.permissions_groups_labels_create_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_labels_destroy(
        self,
        id,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_labels_destroy  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_labels_destroy(id, name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device group label.
            name_or_id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_labels_destroy_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_labels_list(
        self,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_labels_list  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_labels_list(name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):

        Keyword Args:
            page (int): A page number within the paginated result set.. [optional]
            perms (str): Permission types. Only entities with all permissions defined here will be fetched. Defaults to all permissions.. [optional] if omitted the server will use the default value of "view,change,add,delete"
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PaginatedLabelList
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_labels_list_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_labels_partial_update(
        self,
        id,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_labels_partial_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_labels_partial_update(id, name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device group label.
            name_or_id (str):

        Keyword Args:
            patched_resource_permissions (PatchedResourcePermissions): [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            ResourcePermissions
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_labels_partial_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_labels_update(
        self,
        id,
        name_or_id,
        resource_permissions,
        **kwargs
    ):
        """permissions_groups_labels_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_labels_update(id, name_or_id, resource_permissions, async_req=True)
        >>> result = thread.get()

        Args:
            id (int): A unique integer value identifying this device group label.
            name_or_id (str):
            resource_permissions (ResourcePermissions):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['id'] = \
            id
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['resource_permissions'] = \
            resource_permissions
        return self.permissions_groups_labels_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_list(
        self,
        **kwargs
    ):
        """permissions_groups_list  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_list(async_req=True)
        >>> result = thread.get()


        Keyword Args:
            ordering (str): Which field to use when ordering the results.. [optional]
            page (int): A page number within the paginated result set.. [optional]
            search (str): A search term.. [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PaginatedPermissionGroupList
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        return self.permissions_groups_list_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_partial_update(
        self,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_partial_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_partial_update(name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):

        Keyword Args:
            patched_permission_group (PatchedPermissionGroup): [optional]
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_partial_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_retrieve(
        self,
        name_or_id,
        **kwargs
    ):
        """permissions_groups_retrieve  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_retrieve(name_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        return self.permissions_groups_retrieve_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_update(
        self,
        name_or_id,
        permission_group,
        **kwargs
    ):
        """permissions_groups_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_update(name_or_id, permission_group, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):
            permission_group (PermissionGroup):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['permission_group'] = \
            permission_group
        return self.permissions_groups_update_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_users_create(
        self,
        name_or_id,
        user,
        **kwargs
    ):
        """permissions_groups_users_create  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_users_create(name_or_id, user, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):
            user ([User]):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            PermissionGroup
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['user'] = \
            user
        return self.permissions_groups_users_create_endpoint.call_with_http_info(**kwargs)

    def permissions_groups_users_destroy(
        self,
        name_or_id,
        username_or_id,
        **kwargs
    ):
        """permissions_groups_users_destroy  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.permissions_groups_users_destroy(name_or_id, username_or_id, async_req=True)
        >>> result = thread.get()

        Args:
            name_or_id (str):
            username_or_id (str):

        Keyword Args:
            _return_http_data_only (bool): response data without head status
                code and headers. Default is True.
            _preload_content (bool): if False, the urllib3.HTTPResponse object
                will be returned without reading/decoding response data.
                Default is True.
            _request_timeout (int/float/tuple): timeout setting for this request. If
                one number provided, it will be total request timeout. It can also
                be a pair (tuple) of (connection, read) timeouts.
                Default is None.
            _check_input_type (bool): specifies if type checking
                should be done one the data sent to the server.
                Default is True.
            _check_return_type (bool): specifies if type checking
                should be done one the data received from the server.
                Default is True.
            _spec_property_naming (bool): True if the variable names in the input data
                are serialized names, as specified in the OpenAPI document.
                False if the variable names in the input data
                are pythonic names, e.g. snake case (default)
            _content_type (str/None): force body content-type.
                Default is None and content-type will be predicted by allowed
                content-types and body.
            _host_index (int/None): specifies the index of the server
                that we want to use.
                Default is read from the configuration.
            _request_auths (list): set to override the auth_settings for an a single
                request; this effectively ignores the authentication
                in the spec for a single request.
                Default is None
            async_req (bool): execute request asynchronously

        Returns:
            None
                If the method is called asynchronously, returns the request
                thread.
        """
        kwargs['async_req'] = kwargs.get(
            'async_req', False
        )
        kwargs['_return_http_data_only'] = kwargs.get(
            '_return_http_data_only', True
        )
        kwargs['_preload_content'] = kwargs.get(
            '_preload_content', True
        )
        kwargs['_request_timeout'] = kwargs.get(
            '_request_timeout', None
        )
        kwargs['_check_input_type'] = kwargs.get(
            '_check_input_type', True
        )
        kwargs['_check_return_type'] = kwargs.get(
            '_check_return_type', True
        )
        kwargs['_spec_property_naming'] = kwargs.get(
            '_spec_property_naming', False
        )
        kwargs['_content_type'] = kwargs.get(
            '_content_type')
        kwargs['_host_index'] = kwargs.get('_host_index')
        kwargs['_request_auths'] = kwargs.get('_request_auths', None)
        kwargs['name_or_id'] = \
            name_or_id
        kwargs['username_or_id'] = \
            username_or_id
        return self.permissions_groups_users_destroy_endpoint.call_with_http_info(**kwargs)

